'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _includes = require('lodash/includes');

var _includes2 = _interopRequireDefault(_includes);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _isElectron = require('is-electron');

var _isElectron2 = _interopRequireDefault(_isElectron);

var _package = require('./package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Defaults to 'production'
/* eslint max-len: 0 */
/* eslint no-console: 0 */
process.env.NODE_ENV = process.env.NODE_ENV || 'production';

var increaseVerbosityLevel = function increaseVerbosityLevel(val, total) {
    return total + 1;
};

var parseMountPoint = function parseMountPoint(val) {
    val = val || '';

    if (val.indexOf(':') >= 0) {
        var r = val.match(/(?:([^:]*)(?::(.*)))/);
        return {
            url: r[1] || '/static',
            path: r[2]
        };
    }

    return {
        url: '/static',
        path: val
    };
};

var parseController = function parseController(val) {
    val = val ? (val + '').toLowerCase() : '';

    return (0, _includes2.default)(['grbl', 'marlin', 'smoothie', 'smoothieware', 'tinyg', 'g2core'], val) ? val : '';
};

var defaultHost = (0, _isElectron2.default)() ? '127.0.0.1' : '0.0.0.0';
var defaultPort = (0, _isElectron2.default)() ? 0 : 8000;

_commander2.default.version(_package2.default.version).usage('[options]').option('-p, --port <port>', 'Set listen port (default: ' + defaultPort + ')', defaultPort).option('-H, --host <host>', 'Set listen address or hostname (default: ' + defaultHost + ')', defaultHost).option('-b, --backlog <backlog>', 'Set listen backlog (default: 511)', 511).option('-c, --config <filename>', 'Set config file (default: ~/.cncrc)').option('-v, --verbose', 'Increase the verbosity level (-v, -vv, -vvv)', increaseVerbosityLevel, 0).option('-m, --mount [<url>:]<path>', 'Set the mount point for serving static files (default: /static:static)', parseMountPoint, { url: '/static', path: 'static' }).option('-w, --watch-directory <path>', 'Watch a directory for changes').option('--access-token-lifetime <lifetime>', 'Access token lifetime in seconds or a time span string (default: 30d)').option('--allow-remote-access', 'Allow remote access to the server (default: false)').option('--controller <type>', 'Specify CNC controller: Grbl|Marlin|Smoothie|TinyG|g2core (default: \'\')', parseController, '');

_commander2.default.on('--help', function () {
    console.log('  Examples:');
    console.log('');
    console.log('    $ cnc -vv');
    console.log('    $ cnc --mount /pendant:/home/pi/tinyweb');
    console.log('    $ cnc --watch-directory /home/pi/watch');
    console.log('    $ cnc --access-token-lifetime 60d  # e.g. 3600, 30m, 12h, 30d');
    console.log('    $ cnc --allow-remote-access');
    console.log('    $ cnc --controller Grbl');
    console.log('');
});

// Commander assumes that the first two values in argv are 'node' and appname, and then followed by the args.
// This is not the case when running from a packaged Electron app. Here you have the first value appname and then args.
var normalizedArgv = ('' + process.argv[0]).indexOf(_package2.default.name) >= 0 ? ['node', _package2.default.name].concat(process.argv.slice(1)) : process.argv;
if (normalizedArgv.length > 1) {
    _commander2.default.parse(normalizedArgv);
}

var rmDir = function rmDir(dirPath, removeSelf) {
    console.log('del folder ' + dirPath);
    if (removeSelf === undefined) {
        removeSelf = true;
    }

    var files = void 0;
    try {
        files = _fs2.default.readdirSync(dirPath);
        console.log(files);
    } catch (e) {
        return;
    }

    if (files.length > 0) {
        for (var i = 0; i < files.length; i++) {
            var filePath = dirPath + '/' + files[i];
            if (_fs2.default.statSync(filePath).isFile()) {
                _fs2.default.unlinkSync(filePath);
            } else {
                rmDir(filePath);
            }
        }
    }
    if (removeSelf) {
        _fs2.default.rmdirSync(dirPath);
    }
};

var cnc = function cnc() {
    return new _promise2.default(function (resolve, reject) {
        // Change working directory to 'app' before require('./app')
        process.chdir(_path2.default.resolve(__dirname, 'app'));

        // clear _cahce folder
        // https://gist.github.com/liangzan/807712
        rmDir(__dirname + '/web/images/_cache', false);

        // initialize _cache folder
        _fs2.default.createReadStream(__dirname + '/web/images/snap-logo-square-256x256.png').pipe(_fs2.default.createWriteStream(__dirname + '/web/images/_cache/snap-logo-square-256x256.png'));

        _fs2.default.createReadStream(__dirname + '/web/images/snap-logo-square-256x256.png.svg').pipe(_fs2.default.createWriteStream(__dirname + '/web/images/_cache/snap-logo-square-256x256.png.svg'));

        require('./app').createServer({
            port: _commander2.default.port,
            host: _commander2.default.host,
            backlog: _commander2.default.backlog,
            configFile: _commander2.default.config,
            verbosity: _commander2.default.verbose,
            mount: _commander2.default.mount,
            watchDirectory: _commander2.default.watchDirectory,
            accessTokenLifetime: _commander2.default.accessTokenLifetime,
            allowRemoteAccess: !!_commander2.default.allowRemoteAccess,
            controller: _commander2.default.controller
        }, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
            resolve(data);
        });
    });
};

exports.default = cnc;