'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _electron = require('electron');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import AutoUpdater from './AutoUpdater';

var WindowManager = function () {
    function WindowManager() {
        var _this = this;

        (0, _classCallCheck3.default)(this, WindowManager);
        this.windows = [];
        this.title = '';
        this.url = '';

        // https://github.com/electron/electron/blob/master/docs/api/app.md#event-activate-os-x
        // Emitted when the application is activated, which usually happens
        // when the user clicks on the application's dock icon.
        _electron.app.on('activate', function (e) {
            var window = _this.getWindow();
            if (!window) {
                _this.openWindow({
                    title: _this.title,
                    url: _this.url
                });
            }
        });

        // https://github.com/electron/electron/blob/master/docs/api/app.md#event-window-all-closed
        // Emitted when all windows have been closed.
        // This event is only emitted when the application is not going to quit.
        // If the user pressed Cmd + Q, or the developer called app.quit(), Electron
        // will first try to close all the windows and then emit the will-quit event,
        // and in this case the window-all-closed event would not be emitted.
        _electron.app.on('window-all-closed', function () {
            // On OS X it is common for applications and their menu bar
            // to stay active until the user quits explicitly with Cmd + Q
            if (process.platform === 'darwin') {
                var window = _this.getWindow();
                if (window) {
                    // Remember current window attributes that will be used for the next 'activate' event
                    _this.title = window.webContents.getTitle();
                    _this.url = window.webContents.getURL();
                }
                return;
            }

            _electron.app.quit();
        });
    }

    WindowManager.prototype.openWindow = function openWindow(url, options) {
        var _this2 = this;

        var window = new _electron.BrowserWindow((0, _extends3.default)({}, options, {
            show: false
        }));
        var webContents = window.webContents;

        window.on('closed', function (event) {
            var index = _this2.windows.indexOf(event.sender);
            console.assert(index >= 0);
            _this2.windows.splice(index, 1);
        });

        // Open every external link in a new window
        // https://github.com/electron/electron/blob/master/docs/api/web-contents.md
        webContents.on('new-window', function (event, url) {
            event.preventDefault();
            _electron.shell.openExternal(url);
        });

        // Call `ses.setProxy` to ignore proxy settings
        // http://electron.atom.io/docs/latest/api/session/#sessetproxyconfig-callback
        var ses = webContents.session;
        ses.setProxy({ proxyRules: 'direct://' }, function () {
            window.loadURL(url);
            window.show();
        });

        this.windows.push(window);

        // Disable AutoUpdater until an update server is available
        // new AutoUpdater(window);

        return window;
    };

    WindowManager.prototype.getWindow = function getWindow() {
        var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

        if (this.windows.length === 0) {
            return null;
        }
        return this.windows[index] || null;
    };

    return WindowManager;
}(); /* eslint import/no-unresolved: 0 */


exports.default = WindowManager;